from googleapiclient.discovery import build
from google.oauth2 import service_account
from reagan.subclass import Subclass
from io import BytesIO
import pandas as pd
from time import sleep


class SA360(Subclass):
    def __init__(self, version="v2", verbose=0):
        super().__init__(verbose=verbose)
        self.version = version
        self.service_account_filepath = self.get_parameter("sa360").get_parameter("service_account_path"        )
        self._create_service()
        self.dcm_api_calls = 0

    def _create_service(self):

        api_name = "doubleclicksearch"
        self.credentials = service_account.Credentials.from_service_account_file(
            self.service_account_filepath
        )
        self.service = build(api_name, self.version, credentials=self.credentials)

    def get_report_fragments(self, report_id):
        """
        Returns a list containing the fragments (ints) of a report
            - report_id (int): Id of the Report used to make the calls
        """

        request = self.service.reports().get(reportId=report_id)
        report_status = request.execute()
        while True:
            if report_status["isReportReady"]:
                return [
                    report_file["url"].split("/")[-1]
                    for report_file in report_status["files"]
                ]
            request = self.service.reports().get(reportId=report_id)
            report_status = request.execute()
        sleep(60)

    def file_to_df(self, report_id, report_fragment):
        """
        Returns a pandas dataframe given a Report Id and Fragment
            - report_id (int): Id of the Report used to make the calls
            - report_fragment (int): Fragment file of the report
        """

        request = self.service.reports().getFile(
            reportId=report_id, reportFragment=report_fragment
        )
        report_file = request.execute()
        return pd.read_csv(BytesIO(report_file))

    def reports_to_df(self, agency_id, report_type, columns):
        """
        Returns a generator that yields a pandas dataframe with 1000000 rows with the report specifications
            - agency_id (int): Id of the Agency used to make the calls
            - report_type (string): Specified report type
            - columns (list): Columns to include in the report
        """

        body = {
            "reportScope": {"agencyId": agency_id},
            "reportType": report_type,
            "columns": [{"columnName": col} for col in columns],
            "downloadFormat": "csv",
            "maxRowsPerFile": 1000000,
            "statisticsCurrency": "agency",
        }

        # 1. Request Report
        report_request = self.service.reports().request(body=body)
        report = report_request.execute()
        report_id = report["id"]

        # 2. Wait for it to finish
        report_fragments = self.get_report_fragments(report_id)

        # 3. Download files
        for report_fragment in report_fragments:
            yield self.file_to_df(report_id=report_id, report_fragment=report_fragment)

    def decode_error(self, error):
        # Returns a more concise error description
        return eval(error.content.decode())["error"]["message"]


if __name__ == "__main__":
    pass
    # account report
    # report_type = 'campaign'
    # agency_id = 20700000001049589
    # columns = ['campaignId','campaign','campaignStartDate','campaignEndDate']
    # sa = SA360(verbose=1)
    # for df in sa.reports_to_df(report_type = report_type, agency_id = agency_id, columns=columns):
    #     pass



from reagan.subclass import Subclass
from googleapiclient import discovery
from google.oauth2 import service_account


class GCP(Subclass):
    def __init__(self, verbose=0):
        super().__init__(verbose=verbose)
        self.service_account_filepath = self.get_parameter("gcp").get_parameter('service_account_path')
        self.project = self.get_parameter("gcp").get_parameter('project')
        self.region = self.get_parameter("gcp").get_parameter('region')
        self.zone = self.get_parameter("gcp").get_parameter('zone')
        self._create_compute()

    def _create_compute(self):
        SCOPES = ["https://www.googleapis.com/auth/cloud-platform"]
        credentials = service_account.Credentials.from_service_account_file(
            self.service_account_filepath, scopes=SCOPES
        )
        self.compute = discovery.build("compute", "v1", credentials=credentials)

    def create_instance(
        self, name, machine_type, source_disk_image, startup_script=None
    ):

        body = {
            "name": name,
            "machineType": f"zones/{self.zone}/machineTypes/{machine_type}",
            # Specify a network interface with NAT to access the public
            # internet.
            "networkInterfaces": [
                {
                    "network": f"https://www.googleapis.com/compute/v1/projects/{self.project}/global/networks/default",
                    "subnetwork": f"https://www.googleapis.com/compute/v1/projects/{self.project}/regions/{self.region}/subnetworks/default",
                    "name": "nic0",
                    "accessConfigs": [
                        {
                            "type": "ONE_TO_ONE_NAT",
                            "name": "external-nat",
                            "networkTier": "PREMIUM",
                        }
                    ],
                }
            ],
            #
            # # Allow the instance to access cloud storage and logging.
            "serviceAccounts": [
                {
                    "email": "default",
                    "scopes": [
                        "https://www.googleapis.com/auth/devstorage.read_write",
                        "https://www.googleapis.com/auth/logging.write",
                    ],
                }
            ],
        }

        if source_disk_image:
            # Specify the boot disk and the image to use as a source.
            body["disks"] = [
                {
                    "boot": True,
                    "autoDelete": True,
                    "initializeParams": {"sourceImage": source_disk_image},
                }
            ]

        if startup_script:
            # Metadata is readable from the instance and allows you to
            # pass configuration from deployment scripts to instances.
            body["metadata"] = {
                "items": [
                    {
                        # Startup script is automatically executed by the
                        # instance upon startup.
                        "key": "startup-script-url",
                        "value": startup_script,
                    }
                ]
            }

        return (
            self.compute.instances()
            .insert(project=self.project, zone=self.zone, body=body)
            .execute()
        )

    def list_to_df(self):
        request = self.compute.instances().list(project=self.project, zone=self.zone)
        response = request.execute()
        return self._json_to_df(response.get('items'))

    def delete_instance(self, name):
        return self.compute.instances().delete(
            project=self.project,
            zone=self.zone,
            instance=name).execute()

if __name__ == "__main__":
    gcp = GCP()
    # name = "test-instance21"
    # machine_type = "g1-small"
    # source_disk_image = "https://www.googleapis.com/compute/v1/projects/us-gm-175021/global/images/adstxt-image"
    # instance = gcp.create_instance(name=name,machine_type=machine_type,source_disk_image=source_disk_image)
    # images = gcp.list_to_df()
    # deleted_image = gcp.delete_instance(name)



from reagan.ssm_parameter_store import SSMParameterStore
from pandas.io.json import json_normalize
import pandas as pd
import numpy as np
import os


class Subclass(SSMParameterStore):
    def __init__(self, verbose=0):
        super().__init__()
        self.verbose = verbose

    def vprint(self, obj):
        if self.verbose:
            print(obj)
        return

    def _split_dataframe_list(self, df, target_column, separator):
        """ df = dataframe to split,
        target_column = the column containing the values to split
        separator = the symb used to perform the split
        returns: a dataframe with each entry for the target column separated, with each element moved into a new row.
        The values in the other columns are duplicated across the newly divided rows.
        """
        row_accumulator = []

        def _split_list_to_rows(row, separator):
            split_row = row[target_column]
            try:
                for s in split_row:
                    new_row = row.to_dict()
                    new_row[target_column] = s
                    row_accumulator.append(new_row)
            except:
                split_row = [np.nan]
                for s in split_row:
                    new_row = row.to_dict()
                    new_row[target_column] = s
                    row_accumulator.append(new_row)

        df.apply(_split_list_to_rows, axis=1, args=(separator,))
        new_df = pd.DataFrame(row_accumulator)
        return new_df

    def _get_nested_columns(self, df, columns):
        """
        When unnesting the
        """
        nested_cols = {}
        for row in df.iterrows():
            for num, value in enumerate(row[1]):
                col = df.columns.tolist()[num]
                # This behavior means we could potentially want a dict as a final value, but not a list
                if type(value) == list:
                    nested_cols[col] = "list"
                elif col in columns:
                    pass
                elif type(value) == dict:
                    nested_cols[col] = "dict"
        return nested_cols

    def _reduce_columns(self, df, final_columns, sep="_"):

        self.vprint("\nRemoving unused columns")

        current_columns = set(df.columns.tolist())
        columns_to_keep = set()

        for col in final_columns:

            split_col = col.split(sep)
            for i in range(len(split_col)):
                name = "_".join(split_col[0 : i + 1])
                if name in current_columns:
                    columns_to_keep.add(name)

        if self.verbose:
            for col in current_columns:
                action = "Keep" if col in columns_to_keep else "Remove"
                self.vprint(f"{col} - {action}")
        return df[list(columns_to_keep)]

    def _unnest(self, df, columns=None):

        if columns:
            df = self._reduce_columns(df, columns)
        cols = self._get_nested_columns(df, columns)

        while len(cols) > 0:
            for col, typ in cols.items():
                if typ == "list":
                    self.vprint("\nSplitting {0} into separate rows".format(col))
                    df = self._split_dataframe_list(df, col, separator="_")
                elif typ == "dict":
                    self.vprint("\nSplitting {0} into separate columns".format(col))
                    df_new = df[col].apply(lambda x: {} if pd.isnull(x) else x)
                    df_new = df_new.apply(pd.Series)
                    df_new.columns = list(
                        map(lambda x: str(col) + "_" + str(x), df_new.columns.tolist())
                    )
                    df = pd.concat([df, df_new], axis=1).drop(columns=col)
            if columns:
                df = self._reduce_columns(df, columns)
            cols = self._get_nested_columns(df, columns)
        return df

    def _format_query(self, query_input, replacements={}):
        """
        Takes in a string or .sql file and optional 'replacements' dictionary.

        Returns a string containing the formatted sql query and replaces the
        keys in the replacements dictionary with their values.
        """

        # checks if input is a file or query
        if query_input.split(".")[-1] == "sql":
            # print("Reading .sql File")
            f = open(query_input, "r")
            # reading files with a guillemet », add an uncessary Â to the string
            query = f.read().replace("Â", "")
            f.close()
        else:
            query = query_input
        if replacements:
            for key, value in replacements.items():
                query = query.replace(key, str(value))
        return query

    def _json_to_df(self, data, columns=[]):

        # df = json_normalize(data)
        df = pd.DataFrame(data)
        df.columns = [col.replace(".", "_") for col in df.columns.to_list()]
        df = self._unnest(df, columns)

        columns_not_found = set(columns) - set(df.columns)
        self.vprint(
            f'\nColumns not found in the object: {", ".join(columns_not_found)}'
        )

        return df



#!/usr/bin/python
from reagan.subclass import Subclass
from sqlalchemy import create_engine
import pandas as pd
import psycopg2

class PSQL(Subclass):

    def __init__(self, server, verbose=0):
        super().__init__()
        self.verbose = verbose
        self.server = server
        self.connection_string = eval(self.get_parameter("postgres").get_parameter(self.server))
        self._connect_to_database(self.connection_string)

    def _connect_to_database(self,connection):
        '''
        Function called when generating an instance of the class that
        opens the database connection.
        '''

        # hard coding b/c I don't care
        if self.server in ['george','scp']:
            self.conn = create_engine('''postgres+psycopg2://{user}:{password}@{host}/{dbname}'''.format(**connection), connect_args={'sslmode':'prefer'})

        else:
            self.conn = create_engine('''redshift+psycopg2://{user}:{password}@{host}:{port}/{dbname}'''.format(**connection), connect_args={'sslmode':'prefer'})


    def to_df(self,query_input,replacements={}):
        '''
        Takes in a string containing either a correctly formatted SQL
        query, or filepath directed to a .sql file. Returns a pandas DataFrame
        of the executed query.
        '''

        query = self._format_query(query_input,replacements)
        if self.verbose:
            print ('Executing Query:\n\n',format(query,reindent=True,keyword_case='upper'))
        return pd.read_sql(query,self.conn)

    def to_dict(self, schema, table, key, value):
        """
        Executes query and returns results into a dict via sqlalchemy
            - schema (string): schema of the table to pull
            - table (string): table name of the table to pull
            - key (string): column name in the table to be used as the dictionary key
            - value (string): column name in the table to be used as the dictionary value
        """
        df = self.to_df(f'''SELECT DISTINCT {key} AS key, {value} AS value FROM {schema}.{table}''')
        return {row["key"]: row["value"] for idx, row in df.iterrows()}

    def to_list(self, query):
        """
        Executes query and returns results into a list via sqlalchemy
            - query (string): The SQL query to execute. Should be a SELECT
        """
        df = self.to_df(query)
        return df.values.flatten().tolist()

    def execute(self, query, replacements={}):
        """
        Executes query and returns results into a pandas dataframe via sqlalchmey
            - query (string): The SQL query to execute. Should be a SELECT
            - replacements (dict): Modifies the query and replaces any instance of [key] with [value]
        """
        query = self._format_query(query, replacements)
        with self.conn.connect() as con:
            con.execute(query)

    def to_sql(self, df, schema, table, if_exists='fail', index = False, chunksize = 1000):
        '''
        Inserts a pandas dataframe into the specified table in DADL.
        Types accepted are 'append' and 'replace'
        '''

        df.to_sql(
            name=table,
            if_exists=if_exists,
            schema=schema,
            index=index,
            con=self.conn,
            chunksize=chunksize,
        )

    def get_scalar(self, query, replacements={}):
        """
        Executes query and returns a single result
            - query (string): The SQL query to execute. Should be a SELECT
            - replacements (dict): Modifies the query and replaces any instance of [key] with [value]
        """
        query = self._format_query(query, replacements)
        return self.conn.scalar(query)

if __name__ == "__main__":
    q = PSQL('scp')
    # q2 = '''DELETE FROM items.parameters WHERE True'''
    # q1 = '''INSERT INTO items.parameters (name, value_int) VALUES ('max_post_number', 153009616)'''
    # q.execute(q2)
    a = q.to_list('SELECT value_int FROM items.parameters')
    b = q.to_dict(schema = 'items', table = 'parameters', key='name',value = 'value_int')
    c = q.get_scalar('SELECT MAX(value_int) FROM items.parameters')
    # df = p.to_df('''SELECT * FROM carat_gm.dcm_date limit 1000''')


# Copyright (c) 2018 Bao Nguyen <b@nqbao.com>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ==============================================================================

import os
import boto3
from botocore.exceptions import ClientError
import datetime


class SSMParameterStore(object):
    """
    Provide a dictionary-like interface to access AWS SSM Parameter Store
    """
    def __init__(self, prefix=None, ssm_client=None, ttl=None):
        self._prefix = (prefix or '').rstrip('/') + '/'
        self.region = 'us-east-2'
        self._client = boto3.client('ssm',
            region_name=self.region,
            aws_access_key_id=os.environ['AWS_ACCESS_KEY'],
            aws_secret_access_key=os.environ['AWS_SECRET_ACCESS_KEY']
)
        self._keys = None
        self._substores = {}
        self._ttl = ttl

    def get_parameter(self, name, **kwargs):
        assert name, 'Name can not be empty'
        if self._keys is None:
            self.refresh()
        
        abs_key = "%s%s" % (self._prefix, name)
        if name not in self._keys:
            if 'default' in kwargs:
                return kwargs['default']
            
            raise KeyError(name)
        elif self._keys[name]['type'] == 'prefix':
            if abs_key not in self._substores:
                store = SSMParameterStore(prefix=abs_key, ssm_client=self._client, ttl=self._ttl)
                store._keys = self._keys[name]['children']
                self._substores[abs_key] = store
                
            return self._substores[abs_key]
        else:
            return self._get_value(name, abs_key)
    
    def refresh(self):
        self._keys = {}
        self._substores = {}

        paginator = self._client.get_paginator('describe_parameters')
        pager = paginator.paginate(
            ParameterFilters=[
                dict(Key="Path", Option="Recursive", Values=[self._prefix])
            ]
        )

        for page in pager:
            for p in page['Parameters']:
                paths = p['Name'][len(self._prefix):].split('/')
                self._update_keys(self._keys, paths)

    @classmethod
    def _update_keys(cls, keys, paths):
        name = paths[0]
        
        # this is a prefix
        if len(paths) > 1:
            if name not in keys:
                keys[name] = {'type': 'prefix', 'children': {}}

            cls._update_keys(keys[name]['children'], paths[1:])
        else:
            keys[name] = {'type': 'parameter', 'expire': None}

    def keys(self):
        if self._keys is None:
            self.refresh()
            
        return self._keys.keys()
        
    def _get_value(self, name, abs_key):
        entry = self._keys[name]

        # simple ttl
        if self._ttl == False or (entry['expire'] and entry['expire'] <= datetime.datetime.now()):
            entry.pop('value', None)
        
        if 'value' not in entry:
            parameter = self._client.get_parameter(Name=abs_key, WithDecryption=True)['Parameter']
            value = parameter['Value']
            if parameter['Type'] == 'StringList':
                value = value.split(',')

            entry['value'] = value
            
            if self._ttl:
                entry['expire'] = datetime.datetime.now() + datetime.timedelta(seconds=self._ttl)
            else:
                entry['expire'] = None

        return entry['value']
        
    def __contains__(self, name):
        try:
            self.get(name)
            return True
        except:
            return False

    def __getitem__(self, name):
        return self.get(name)

    def __setitem__(self, key, value):
        raise NotImplementedError()
    
    def __delitem__(self, name):
        raise NotImplementedError()
        
    def __repr__(self):
        return 'ParameterStore[%s]' % self._prefix


#!/usr/bin/python
from reagan.subclass import Subclass
import pandas as pd
import pyodbc
import sqlalchemy
import urllib


class SQLServer(Subclass):
    def __init__(self, server):
        super().__init__()
        self.server = server
        self._connect_to_database(self.get_parameter("sqlserver").get_parameter(self.server))

    def _connect_to_database(self, connection):

        # Connect via pyodbc. Used for DML statements
        self.conn = pyodbc.connect(connection)

        # conn.cursor will return a cursor object, you can use this cursor to perform queries
        self.cursor = self.conn.cursor()

        # Connect via sqlalchemy to perform interactions with the pandas
        conn_string = "mssql+pyodbc:///?odbc_connect={}".format(
            urllib.parse.quote_plus(connection)
        )
        self.engine = sqlalchemy.create_engine(
            conn_string, echo=False, fast_executemany=True
        )

    def to_df(self, query, replacements={}, table=None):
        """
        Executes query and returns results into a pandas dataframe via sqlalchemy
            - query (string): The SQL query to execute. Should be a SELECT
            - replacements (dict): Modifies the query and replaces any instance of [key] with [value]
        """
        if table:
            query = f'SELECT * FROM {table}'
        else:
            query = self._format_query(query, replacements)
        return pd.read_sql(query, self.conn)

    def to_list(self, query):
        """
        Executes query and returns results into a list via sqlalchemy
            - query (string): The SQL query to execute. Should be a SELECT
        """
        df = self.to_df(query)
        return df.values.flatten().tolist()

    def to_dict(self, schema, table, key, value):
        """
        Executes query and returns results into a dict via sqlalchemy
            - schema (string): schema of the table to pull
            - table (string): table name of the table to pull
            - key (string): column name in the table to be used as the dictionary key
            - value (string): column name in the table to be used as the dictionary value
        """
        df = self.to_df(
            f"SELECT DISTINCT {key} AS [key], {value} AS [value] FROM {schema}.{table}"
        )
        return {row["key"]: row["value"] for idx, row in df.iterrows()}

    def to_sql(self, df, name, schema, if_exists="fail", index=False, chunksize=1000):
        """
        Inserts a pandas dataframe into the specified table in DADL.
            - df (DataFrame): The data which to insert into the SQL table.
            - name (string): The table name to insert data into.
            - schema (string): The schema location of the table.
            - if_exists (string): How to treat the insertion. Accepted values are: 'fail','append','replace' (default is fail).
                - If 'replace' is selected, the table will be dropped and recreated with auto-assigned datatypes.
            - index (string): Whether to include the DataFrame index in the table (default is False).
            - chunksize (int): Chunksize will determine how many rows are written to the database at a time.
        """

        df.to_sql(
            name=name,
            if_exists=if_exists,
            schema=schema,
            index=index,
            con=self.engine,
            chunksize=chunksize,
        )

    def execute(self, query, replacements={}):
        """
        Executes query and returns results into a pandas dataframe via sqlalchmey
            - query (string): The SQL query to execute. Should be a SELECT
            - replacements (dict): Modifies the query and replaces any instance of [key] with [value]
        """
        query = self._format_query(query, replacements)
        try:
            self.cursor.execute(query)
        except pyodbc.DatabaseError as e:
            print(e)
            self.conn.rollback()
        else:
            self.conn.commit()

    def get_scalar(self, query, replacements={}):
        """
        Executes query and returns a single result
            - query (string): The SQL query to execute. Should be a SELECT
            - replacements (dict): Modifies the query and replaces any instance of [key] with [value]
        """
        query = self._format_query(query, replacements)
        return self.engine.scalar(query)


if __name__ == "__main__":
    ss = SQLServer("102")
    sites = ss.to_dict(
        schema="glops", table="DCM_Sites", key="Site_Id_DCM", value="Site_DCM"
    )




import pandas as pd
from configparser import ConfigParser
from reagan.subclass import Subclass
import smartsheet


class SmartsheetAPI(Subclass):
    def __init__(self):
        super().__init__()
        self.bearer_token = self.get_parameter("smartsheets").get_parameter("bearer_token")
        self.conn = smartsheet.Smartsheet(self.bearer_token)

    def get_attachment_url(self, sheet_id, attachment_id):
        """
        Inputs - sheet_id (int), attachment_id (int)
        Returns - url of attachment generated by smartsheet API. (Only lasts for 2min)
        """

        attachments_obj = self.conn.Attachments.get_attachment(sheet_id, attachment_id)
        a = attachments_obj.to_dict()
        if a.get("url"):
            return a.get("url")
        else:
            self.vprint("No url returned: {a}")

    def attachment_to_dict(self, sheet_id, attachment_id):
        """
        Use only for excel attachments

        Inputs - sheet_id (int), attachment_id (int)
        Returns - Dictionary Object: key = Sheet Name, value = DataFrame 
        """

        url = self.get_attachment_url(sheet_id, attachment_id)
        excel_sheet_names = pd.ExcelFile(url).sheet_names
        attachment = {}
        for sheet_name in excel_sheet_names:
            df = pd.read_excel(url, sheet_name=sheet_name)
            attachment[sheet_name] = df
        return attachment

    def discussions_to_df(self, sheet_id):
        discussions_obj = self.conn.Discussions.get_all_discussions(
            sheet_id, include_all=True, include=["comments", "attachments"]
        )
        data = discussions_obj.to_dict()

        # Check that data exists else return empty df
        if "data" not in data.keys():
            return pd.DataFrame()

        disc = self._json_to_df(data["data"])
        disc["sheet_id"] = sheet_id

        return disc

    def sheet_to_df(self, sheet_id):
        # Make API Call
        sheet_obj = self.conn.Sheets.get_sheet(sheet_id)

        # Sheet columns to use
        other_df_cols = ["createdAt", "id", "modifiedAt", "name", "permalink"]

        # Get the data in dict form
        data = sheet_obj.to_dict()

        # Check that data exists else return empty df
        for x in ["columns", "rows"]:
            if x not in data.keys():
                return pd.DataFrame()

        # Get the columns first
        columns_dict = {}
        columns_dict["columns"] = data["columns"]
        columns_df = self._json_to_df(columns_dict)
        columns_df = columns_df[["columns_id", "columns_title"]].drop_duplicates()

        # Get the acutal data
        rows_dict = {}
        rows_dict["rows"] = data["rows"]
        rows_df = self._json_to_df(rows_dict)

        df = pd.merge(
            rows_df,
            columns_df,
            left_on="rows_cells_columnId",
            right_on="columns_id",
            how="left",
        )

        for col in other_df_cols:
            df[col] = data[col]

        return df



from reagan.subclass import Subclass
import requests
from bs4 import BeautifulSoup
from textblob import TextBlob
import pandas as pd


class Ihub(Subclass):
    
    def __init__(self):
        self.headers = {
            "User-Agent": 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36'
        }

    def page_response(self, message_id, timeout = 10):
        '''
        Use the requests and beautifulsoup modules to make the request to the website
        '''
        payload = {'message_id' : message_id}
        url = f"https://investorshub.advfn.com/boards/read_msg.aspx"
        r = requests.get(url=url, timeout=timeout, headers=self.headers, params = payload)
        return BeautifulSoup(r.content, "lxml")

    def get_message_data(self, message_id):
        """ Web scrapes post information from the specified message_id

        Parameters: message_id (int): The message number to extract data from
        """

        site_id_codes = {
            "active": { 
                # This is where the data from the post will originate from
                'message' : "ctl00_CP1_mbdy_dv"
            
                # Where to extract the post number for the specific board the message is on
                ,'post_number' : "ctl00_CP1_mh1_tbPost"
            
                # Finds the date the message was posted
                ,'message_date' : "ctl00_CP1_mh1_lblDate"
            
                # extracting the code of the specific ticker or board
                ,'ihub_code' : "ctl00_CP1_bbc1_hlBoard"
            }
            ,"error": { 
                # If this id has any data, it means the post is erroneous
                'missing_post' : "ctl00_CP1_L1"

                # If a message was deleted, the message will appear here
                ,'deleted_post' : 'ctl00_CP1_na'
            }
        }

        try:
            soup = self.page_response(message_id)
        except requests.exceptions.Timeout:
            return {}

        output = {'message_id': message_id, 'status': 'Active'}

        for id_type, id_code in site_id_codes['error'].items():
            data = soup.find(id=id_code)
            if data:
                if id_type == 'missing_post':
                    output['status'] = 'Error'
                    output['error_message'] = data.text.strip()
                    return output
                if id_type == 'deleted_post':
                    output['status'] = 'Error'
                    output['error_message'] = data.text.strip()
                    return output

        for id_type, id_code in site_id_codes['active'].items():
            data = soup.find(id=id_code)
            if data:
                if id_type == 'message':
                    body = data.text.strip()
                    tb = TextBlob(body)
                    output['sentiment_polarity'], output['sentiment_subjectivity'] = tb.sentiment
                if id_type == 'post_number':
                    output['post_number'] = int(data.get('value',0))
                if id_type == 'message_date':
                    output['message_date'] = pd.to_datetime(data.text)
                if id_type == 'ihub_code':
                    output['ihub_code'] = data["href"].replace("/", "")
        return output


if __name__ == "__main__":
    pass
    # deleted post
    # message_id = 144689607
    # good post
    # message_id = 144689616
    # non-existant-post
    # message_id=254689616
    # other board
    # message_id = 144802665
    # message_id = 77864629
    message_id = 1309083

    s = Ihub()
    z = s.page_response(message_id)